import 'dart:convert';

import 'package:car_app/converter/storage_converters/route.dart';
import 'package:car_app/model/route.dart';
import 'package:car_app/service/interface/key_value_storage.dart';

class RouteStorageService extends KeyValueStorageService<String, List<Route>> {
  final KeyValueStorageService<String, String> storageService;

  RouteStorageService({required this.storageService});

  @override
  Future<void> add(String key, List<Route> value) {
    var json = jsonEncode(value.map((e) => e.toMap()).toList());
    return storageService.add(key, json);
  }

  @override
  Future<List<Route>> get(String key) async {
    var json = await storageService.get(key);
    return _getRoutesFromJson(json);
  }

  @override
  Future<List<Route>> remove(String key) async {
    var json = await storageService.remove(key);
    return _getRoutesFromJson(json);
  }

  List<Route> _getRoutesFromJson(String json) {
    var routeMaps = List<Map<String, dynamic>>.from(jsonDecode(json));
    return routeMaps.map((e) => RouteMapConverter.fromMap(e)).toList();
  }
}
