import 'package:car_app/view/defaults.dart';
import 'package:car_app/view/routes_titles.dart';
import 'package:car_app/viewmodel/authentication.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class AuthenticationView extends StatefulWidget {

  const AuthenticationView({super.key});
  
  @override
  State<StatefulWidget> createState() => _AuthenticationViewState();
}

class _AuthenticationViewState extends State<AuthenticationView> {
  final TextEditingController _nameController = TextEditingController();
  final TextEditingController _usernameController = TextEditingController();
  final TextEditingController _passwordController = TextEditingController();

  late final String _usernameFieldName;

  bool _isLogin = true;
  bool _hasError = false;
  String _errorText = "";
  String get _usernameAutofillHint => _isLogin ? AutofillHints.username : AutofillHints.newUsername;
  String get _passwordAutofillHint => _isLogin ? AutofillHints.password : AutofillHints.newPassword;

  set isLogin(bool value) => setState(() => _isLogin = value);
  set hasError(bool value) => setState(() => _hasError = value);
  set errorText(String value) => setState(() => _errorText = value);

  @override
  void initState() {
    super.initState();
    var viewModel = Provider.of<AuthenticationViewModel>(context, listen: false);

    setState(() {
      _usernameFieldName = viewModel.authenticationService.usernameFieldName;
    });
  }

  void popPage(bool isNotCancelled) {
    Navigator.pop(context, isNotCancelled);
  }
  
  @override
  Widget build(BuildContext context) {
    return Consumer<AuthenticationViewModel>(
      builder: (_, viewModel, __){
        return Scaffold(
          backgroundColor: getBackgroundColor(context),
          appBar: DefaultAppBar(
            context,
            titleText: AuthenticationRouting.title,
          ),
          body: Consumer<AuthenticationViewModel>(
            builder: (_, viewModel, __) {
              return Center(
                child: Padding(
                  padding: const EdgeInsets.symmetric(
                    horizontal: 25,
                    vertical: 20
                  ),
                  child: Column(
                    children: [
                      Column(
                        children: [
                          FractionallySizedBox(
                            widthFactor: 0.3,
                            child: Image.asset(viewModel.authenticationService.logoPath),
                          ),
                          Text(
                            viewModel.authenticationService.providerName,
                            textScaler: const TextScaler.linear(1.5),
                          ),
                        ],
                      ),

                      Expanded(
                        flex: 1,
                        child: SingleChildScrollView(
                          child: Column(
                            children: [
                              const SizedBox(height: 60),
                              Visibility(
                                visible: !_isLogin,
                                maintainSize: false,
                                child: Column(
                                  children: [
                                    TextField(
                                      controller: _nameController,
                                      decoration: const InputDecoration(
                                        border: OutlineInputBorder(),
                                        label: Text("Name"),
                                      ),
                                    ),
                                    const SizedBox(height: 10),
                                  ],
                                )
                              ),

                              AutofillGroup(
                                child: Column(
                                  children: [
                                    TextField(
                                      controller: _usernameController,
                                      autofillHints: [_usernameAutofillHint],
                                      decoration: InputDecoration(
                                        border: const OutlineInputBorder(),
                                        label: Text(_usernameFieldName),
                                      ),
                                    ),

                                    const SizedBox(height: 10),

                                    TextField(
                                      controller: _passwordController,
                                      obscureText: true,
                                      autofillHints: [_passwordAutofillHint],
                                      decoration: const InputDecoration(
                                        border: OutlineInputBorder(),
                                        label: Text("Password"),
                                      ),
                                    ),
                                  ],
                                )
                              ),

                              Visibility(
                                visible: _hasError,
                                maintainSize: true,
                                maintainAnimation: true,
                                maintainState: true,
                                child: Text(_errorText),
                              ),

                              const SizedBox(height: 20),

                              Visibility(
                                visible: _isLogin,
                                maintainSize: false,
                                child: TextButton(
                                  onPressed: () => isLogin = false,
                                  child: const Text('Create an account')
                                ),
                              ),

                              Visibility(
                                visible: !_isLogin,
                                maintainSize: false,
                                child: TextButton(
                                  onPressed: () => isLogin = true,
                                  child: const Text('Login'),
                                ),
                              )
                            ],
                          ),
                        ),
                      ),
                      
                      const Expanded(
                        flex: 0,
                        child: SizedBox(),
                      ),

                      Row(
                        children: [
                          Expanded(
                            child: OutlinedButton(
                              child: const Text("Cancel"),
                              onPressed: () => popPage(false),
                            )
                          ),
                          const SizedBox(width: 80),
                          Visibility(
                            visible: _isLogin,
                            maintainSize: false,
                            child: Expanded(
                              child: OutlinedButton(
                                child: const Text("Login"),
                                onPressed: () {
                                  hasError = false;
                                  viewModel.login(
                                    _usernameController.text,
                                    _passwordController.text,
                                    () => popPage(true), 
                                    (error) {
                                      errorText = error;
                                      hasError = true;
                                    }
                                  );
                                },
                              )
                            ),
                          ),
                          Visibility(
                            visible: !_isLogin,
                            maintainSize: false,
                            child: Expanded(
                              child: OutlinedButton(
                                child: const Text("Register"),
                                onPressed: () {
                                  hasError = false;
                                  viewModel.register(
                                    _nameController.text,
                                    _usernameController.text,
                                    _passwordController.text,
                                    () => popPage(true),
                                    (error) {
                                      errorText = error;
                                      hasError = true;
                                    }
                                  );
                                },
                              ),
                            ),
                          ),
                        ],
                      ),
                    ],
                  ),
                )
              );
            },
          ),
        );
      }
    );
  }
}
